@@ -37,6 +37,7 @@
 #include <util.h>
 #include <message.h>
 #include <csv.h>
+#include <picofont.h>
 
 u_int gFontsWidth = 0;
 u_int gFontsHeight = 0;
@@ -44,7 +45,114 @@ u_int gFontsHeight = 0;
 u_char sgFontsDefaultGlyph[] = {0x80};
 u_char sgFontsDefaultDGlyph[] = {0xC0};
 
+#define FMACRO_94__FONT(final, aline, name) {\
+	tfont_default_glyph, name, 1, 1,\
+	TFONT_FT_SINGLE | TFONT_FT_94CHAR| final,\
+	aline, TFONT_ALIAS, NULL, sgFontsDefaultGlyph, NULL,\
+	0xFF, 0x00, 0xFF, 0x00, 0x00, 2, 1 }
+
+#define FMACRO_96__FONT(final, aline, name) {\
+	tfont_default_glyph, name, 1, 1,\
+	TFONT_FT_SINGLE | TFONT_FT_96CHAR| final,\
+	aline, TFONT_ALIAS, NULL, sgFontsDefaultGlyph, NULL,\
+	0xFF, 0x00, 0xFF, 0x00, 0x00, 2, 1 }
+
+#define FMACRO_94N_FONT(final, aline, name) {\
+	tfont_default_glyph, name, 2, 1,\
+	TFONT_FT_DOUBLE | TFONT_FT_94CHAR| final,\
+	aline, TFONT_ALIAS, NULL, sgFontsDefaultDGlyph, NULL,\
+	0xFF, 0x00, 0xFF, 0x00, 0x00, 2, 2 }
+
 TFont gFont[] = {
+	FMACRO_94__FONT(0x40, FH_LEFT , "iso646-1973irv"),
+
+	/* CJK Character sets */
+	FMACRO_94N_FONT(0x40, FH_LEFT , "jisc6226-1978"),
+	FMACRO_94N_FONT(0x41, FH_LEFT , "gb2312-80"),
+	FMACRO_94N_FONT(0x42, FH_LEFT , "jisx0208-1983"),
+#if 0
+	FMACRO_94N_FONT(0x42, FH_LEFT , "jisx0208-1990"),
+#endif
+	FMACRO_94N_FONT(0x43, FH_LEFT , "ksc5601-1987"),
+	FMACRO_94N_FONT(0x44, FH_LEFT , "jisx0212-1990"),
+	FMACRO_94N_FONT(0x45, FH_LEFT , "ccitt-extended-gb" /*aka iso-ir-165*/),
+	FMACRO_94N_FONT(0x47, FH_LEFT , "cns11643-1992.1"),
+	FMACRO_94N_FONT(0x48, FH_LEFT , "cns11643-1992.2"),
+	FMACRO_94N_FONT(0x49, FH_LEFT , "cns11643-1992.3"),
+	FMACRO_94N_FONT(0x4a, FH_LEFT , "cns11643-1992.4"),
+	FMACRO_94N_FONT(0x4b, FH_LEFT , "cns11643-1992.5"),
+	FMACRO_94N_FONT(0x4c, FH_LEFT , "cns11643-1992.6"),
+	FMACRO_94N_FONT(0x4d, FH_LEFT , "cns11643-1992.7"),
+	/* Domestic ISO 646 Character sets */
+	FMACRO_94__FONT(0x41, FH_LEFT , "bs4730"),
+	FMACRO_94__FONT(0x42, FH_LEFT , "ansix3.4-1968" /* aka ASCII */),
+	FMACRO_94__FONT(0x43, FH_LEFT , "nats1-finland-sweden"),
+	FMACRO_94__FONT(0x47, FH_LEFT , "sen850200b"),
+	FMACRO_94__FONT(0x48, FH_LEFT , "sen850200c"),
+	FMACRO_94__FONT(0x4a, FH_LEFT , "jisc6220-1969roman"),
+	FMACRO_94__FONT(0x59, FH_LEFT , "uni0204-70"),
+	FMACRO_94__FONT(0x4c, FH_LEFT , "olivetti-portuguese"),
+	FMACRO_94__FONT(0x5a, FH_LEFT , "olivetti-spanish"),
+	FMACRO_94__FONT(0x4b, FH_LEFT , "din66003"),
+	FMACRO_94__FONT(0x52, FH_LEFT , "nfz62-010-1973"),
+	FMACRO_94__FONT(0x54, FH_LEFT , "gb1988-80"),
+	FMACRO_94__FONT(0x60, FH_LEFT , "ns4551-1"),
+	FMACRO_94__FONT(0x61, FH_LEFT , "ns4551-2"),
+	FMACRO_94__FONT(0x66, FH_LEFT , "nfz62-010-1982"),
+	FMACRO_94__FONT(0x67, FH_LEFT , "ibm-portuguese"),
+	FMACRO_94__FONT(0x68, FH_LEFT , "ibm-spanish"),
+	FMACRO_94__FONT(0x69, FH_LEFT , "msz7795.3"),
+	FMACRO_94__FONT(0x6e, FH_LEFT , "jisc6229-1984ocr-b"),
+	FMACRO_94__FONT(0x75, FH_LEFT , "ccitt-t.61-1"),
+	FMACRO_94__FONT(0x77, FH_LEFT , "csaz243.4-1985alt1-1"),
+	FMACRO_94__FONT(0x78, FH_LEFT , "csaz243.4-1985alt1-2"),
+	FMACRO_94__FONT(0x7a, FH_LEFT , "jusi.b1.002"),
+#if 0
+	/* Domestic ISO 646 Character sets with I-oct */
+	FMACRO_94__FONT(0x41, FH_LEFT , "nc99-10:81"),
+	FMACRO_94__FONT(0x42, FH_LEFT , "iso646-1992invariants"),
+#endif
+	/* Quasi ISO 646 Character sets */
+	FMACRO_94__FONT(0x45, FH_LEFT , "nats1-denmark-norway"),
+	FMACRO_94__FONT(0x55, FH_LEFT , "honeywell-bull-latin-greek"),
+	FMACRO_94__FONT(0x56, FH_LEFT , "british-post-office-teletext"),
+	FMACRO_94__FONT(0x57, FH_LEFT , "inis-irv-subset"),
+	/* Greek Character sets */
+	FMACRO_94__FONT(0x5b, FH_LEFT , "olivetti-greek"),
+	FMACRO_94__FONT(0x5c, FH_LEFT , "olivetti-latin-greek"),
+	FMACRO_94__FONT(0x58, FH_LEFT , "iso5428-1974"),
+	FMACRO_94__FONT(0x53, FH_LEFT , "iso5428-1980"),
+#if 0
+	/* Greek Character sets  with I-oct */
+	FMACRO_94__FONT(0x40, FH_LEFT , "ccitt-greek1"),
+#endif
+	/* Cyrillic Character sets */
+	FMACRO_94__FONT(0x4e, FH_LEFT , "iso5427-1981"),
+	FMACRO_94__FONT(0x5e, FH_LEFT , "inis-cyrillic-extension"),
+	FMACRO_94__FONT(0x51, FH_LEFT , "iso5427-1981extension"),
+	FMACRO_96__FONT(0x40, FH_LEFT , "ecma-94cyrillic"),
+	FMACRO_94__FONT(0x7b, FH_LEFT , "jusi.b1.003"),
+	FMACRO_94__FONT(0x7d, FH_LEFT , "jusi.b1.004"),
+	FMACRO_96__FONT(0x4f, FH_LEFT , "stsev358-88"),
+	/* ISO 8859 variants */
+	FMACRO_96__FONT(0x41, FH_RIGHT, "iso8859.1-1987"),
+	FMACRO_96__FONT(0x42, FH_RIGHT, "iso8859.2-1987"),
+	FMACRO_96__FONT(0x43, FH_RIGHT, "iso8859.3-1988"),
+	FMACRO_96__FONT(0x44, FH_RIGHT, "iso8859.4-1988"),
+	FMACRO_96__FONT(0x4c, FH_RIGHT, "iso8859.5-1988"),
+	FMACRO_96__FONT(0x47, FH_RIGHT, "iso8859.6-1987"),
+	FMACRO_96__FONT(0x46, FH_RIGHT, "iso8859.7-1987"),
+	FMACRO_96__FONT(0x48, FH_RIGHT, "iso8859.8-1988"),
+	FMACRO_96__FONT(0x4d, FH_RIGHT, "iso8859.9-1989"),
+	FMACRO_96__FONT(0x56, FH_RIGHT, "iso8859.10-1992"),
+	/* CCITT T.101 Mosaic Charactor Sets */
+	FMACRO_94__FONT(0x79, FH_LEFT , "ccitt-mosaic-1"),
+	FMACRO_96__FONT(0x7d, FH_LEFT , "ccitt-mosaic-sup1"),
+	FMACRO_94__FONT(0x63, FH_LEFT , "ccitt-mosaic-sup2"),
+	FMACRO_94__FONT(0x64, FH_LEFT , "ccitt-mosaic-sup3"),
+	/* Japano */
+	FMACRO_94__FONT(0x49, FH_LEFT , "jisc6220-1969kana"),
+#if 0
 					/* 1st element shall be ASCII */
 	{ tfont_default_glyph,	"ASCII", 1, 1,
 		TFONT_FT_SINGLE|TFONT_FT_94CHAR| 0x42,
@@ -152,6 +260,7 @@ TFont gFont[] = {
 		FH_LEFT, TFONT_ALIAS, NULL, sgFontsDefaultDGlyph, NULL,
 		0xFF, 0x00, 0xFF, 0x00, 0x00, 1, 1
 	},
+#endif
 	{	tfont_default_glyph,	NULL, 1, 1,
 		0x00000000,
 		FH_LEFT, TFONT_ALIAS, NULL, sgFontsDefaultDGlyph, NULL,
@@ -159,6 +268,32 @@ TFont gFont[] = {
 	},
 };
 
+void tfont_ary_show_list(FILE* fp)
+{
+	TFont* ce = gFont;
+	char c;
+	u_int n;
+	u_int u;
+	u_int d;
+	u_int f1;
+	u_int f2;
+	while (ce->name) {
+		n = ce->fsignature;
+		c = n & 255;
+		u = (n >> 4) & 15;
+		d = (n >> 0) & 15;
+		f1 = n & TFONT_FT_DOUBLE;
+		f2 = n & TFONT_FT_96CHAR;
+		fprintf(fp, " [%02u/%02u(%c) %s%s] %s\n",
+			u, d, c,
+			f2 ? "96" : "94",
+			f1 ? "^N" : "  ",
+			ce->name);
+		ce++;
+	}
+	
+}
+
 void tfont_ary_final(void)
 {
 	TFont* ce = gFont;
@@ -202,9 +337,48 @@ void tfont_final(TFont* p)
 
 const u_char* tfont_default_glyph(
 	TFont* p,
-	u_char b1,
-	u_char b2)
+	u_int chlw)
 {
+	static u_char gly[PICOFONT_HEIGHT * 2];
+
+	u_char* cp = gly;
+	u_int i;
+	u_int a;
+	u_int b;
+
+	*cp++ = 0x80;
+	*cp++ = 0x00;
+	for (i = 0 ; i < 16 ; i+=4) {
+		b = (chlw >> (12-i)) & 0xf;
+		b <<= 1;
+		a = (chlw >> (28-i)) & 0xf;
+		a <<= 1;
+		*cp++ = 0x80 | gPicoFontLeft[b++];
+		*cp++ = gPicoFontLeft[a++];
+		*cp++ = 0x80 | gPicoFontLeft[b];
+		*cp++ = gPicoFontLeft[a];
+		*cp++ = 0x80;
+		*cp++ = 0x00;
+	}
+	*cp++ = 0x80;
+	*cp++ = 0x00;
+	if (p->bytec == 2) {
+		*cp++ = 0xff;
+		*cp = 0xfe;
+	} else {
+		*cp++ = 0xfe;
+		*cp = 0x00;
+	}
+
+	return gly;
+}
+
+const u_char* tfont_standard_glyph(
+	TFont* p,
+	u_int chlw)
+{
+	u_char b2 = (chlw >> 8) & 0xFF;
+	u_char b1 = chlw & 0xFF;
 	u_int i;
 
 	if (b1 < p->colf || p->coll < b1 || b2 < p->rowf || p->rowl < b2) {
@@ -253,6 +427,7 @@ static void tfont_setup_font(TFont* pf, const char* fname, FONTSET_HALF hf)
 
 	tpcf_as_tfont(&pcf, pf);
 	pf->fhalf = hf;
+	pf->conv = tfont_standard_glyph;
 
 	tpcf_final(&pcf);
 }
@@ -280,6 +455,7 @@ static void tfont_fontlist_glyph_size(void)
 {
 	TFont* p = gFont;
 	u_int w;
+	u_int dw;
 
 	while (p->name) {
 		if (gFontsHeight < p->height) {
@@ -292,6 +468,19 @@ static void tfont_fontlist_glyph_size(void)
 		}
 		p++;
 	}
+
+	if (PICOFONT_WIDTH <= gFontsWidth && PICOFONT_HEIGHT <= gFontsHeight) {
+		p = gFont;
+		while (p->name) {
+			if (p->conv == tfont_default_glyph) {
+				dw = p->width;
+				p->height = PICOFONT_HEIGHT;
+				p->width = PICOFONT_WIDTH * dw;
+			}
+			p++;
+		}
+	}
+
 }
 
 int tfont_is_valid(TFont* p)
