@@ -41,6 +41,7 @@
 #include <signal.h>
 #include <fcntl.h>
 #include <sys/stat.h>
+#include <getopt.h>
 
 
 #include <pcf.h>
@@ -51,6 +52,8 @@
 #include <message.h>
 #include <mytypes.h>
 #include <getcap.h>
+#include <util.h>
+#include <main.h>
 
 
 #ifdef SYSCONFDIR
@@ -61,42 +64,41 @@
 
 TFrameBufferMemory gFramebuffer;
 
-#if 0
-static void tf_get_options(int argc, const char *argv[])
+static void tapp_get_options(TApplication* p, int argc, char *argv[])
 {
+	static struct option optList[] = {
+		{"shell",	1, NULL, 'e'},
+		{"exec",	1, NULL, 'e'},
+		{"help",	0, NULL, 'h'},
+		{NULL,		0, NULL, 0},
+	};
+	int c;
+	int ix;
 	int i;
-	struct TF_FontList *flist = &tf_fontlist;
-
-	memset((void*)flist, 0, sizeof(*flist));
-	for (i = 1 ; i < argc ; i++) {
-		if (strncmp("--font=", argv[i], 7) == 0) {
-			char *fp;
-			int len = strlen(argv[i]+7);
-			if((fp = (char*)malloc(len+1)) == NULL)
-				die("get_options: %s\n",strerror(errno));
-			strncpy((void*)fp,argv[i]+7,len);
-			*(fp+len) = '\0';
-			flist->font = fp;
-			if((flist->next = (struct TF_FontList*)
-					malloc(sizeof(*flist))) == NULL)
-				die("get_options: %s\n",strerror(errno));
-			flist = flist->next;
-			memset((void*)flist, 0, sizeof(*flist));
+	
+	while (1) {
+		c = getopt_long(argc, argv, "e:h", optList, &ix);
+		if (c == EOF) {
+			break;
+		}
+		switch (c) {
+		case 'e':
+			p->gExecShell = optarg;
+			break;
+		case 'h':
+			p->gOptShowHelpQ = TRUE;
+			break;
+		default:
+			break;
 		}
-		if (strncmp("--gamma=", argv[i], 8) == 0)
-			fbgamma = (float)atof(argv[i] + 8);
 	}
+	p->gExecShellArgv = (char**)calloc(argc + 2 - optind, sizeof(char*));
+	p->gExecShellArgv[0] = p->gExecShell;
+	for (i = 0 ; i < argc+1 - optind ; i++) {
+		p->gExecShellArgv[i+1] = argv[i + optind];
+	}
+	
 }
-#endif
-
-typedef struct Raw_TApplication {
-	TBool gOrigVirtualConsoleQ;
-	int gOrigVirtualConsole;
-	TBool gCapsQ;
-	TCaps gCaps;
-} TApplication;
-
-static TApplication gApp;
 
 void tapp_change_to_original_console(TApplication* p)
 {
@@ -126,8 +128,11 @@ void tapp_final(TApplication* p)
 	if (p->gOrigVirtualConsoleQ) {
 		tapp_change_to_original_console(p);
 	}
+	util_free(p->gExecShellArgv);
 }
 
+TApplication gApp;
+
 void tapp_final_at_exit(void)
 {
 	tapp_final(&gApp);
@@ -135,10 +140,20 @@ void tapp_final_at_exit(void)
 
 void tapp_init(TApplication* p)
 {
+	static char shell[128];
+	
 	p->gOrigVirtualConsoleQ	= FALSE;
 	p->gOrigVirtualConsole	= -1;
 	tcaps_init(&(p->gCaps));
 	p->gCapsQ	= TRUE;
+	p->gOptShowHelpQ = FALSE;
+	if (getenv("SHELL")) {
+		strcpy(shell, getenv("SHELL"));
+		p->gExecShell = shell;
+	} else {
+		p->gExecShell = "/bin/sh";
+	}
+	p->gExecShellArgv = NULL;
 
 	atexit(tapp_final_at_exit);
 }
@@ -199,22 +214,106 @@ void tapp_change_to_new_console(TApplication* p)
         dup2(vfd, 2);
 }
 
+void ShowCaps(void)
+{
+	fprintf(stderr,
+		"======== Capabilities =====================================\n"
+		"[[ COLOR ]]\n"
+#ifdef JFB_VGA16FB
+			" VGA16"
+#endif
+#ifdef JFB_PSEUDOCOLOR
+			" Pseudo"
+#endif
+#ifdef JFB_DIRECTCOLOR
+			" Direct"
+#endif
+#ifdef JFB_TRUECOLOR
+			" True"
+#endif
+		"\n"
+		"[[ BITS ]]\n"
+#ifdef JFB_1BPP
+			" 1bpp"
+#endif
+#ifdef JFB_2BPP
+			" 2bpp"
+#endif
+#ifdef JFB_4BPP
+			" 4bpp"
+#endif
+#ifdef JFB_VGA16FB
+			" 4bpp(VGA)"
+#endif
+#ifdef JFB_8BPP
+			" 8bpp"
+#endif
+#ifdef JFB_15BPP
+			" 15bpp"
+#endif
+#ifdef JFB_16BPP
+			" 16bpp"
+#endif
+#ifdef JFB_24BPP
+			" 24bpp"
+#endif
+#ifdef JFB_32BPP
+			" 32bpp"
+#endif
+		"\n"
+	);
+	fprintf(stderr, "[[ FONTSET ]]\n");
+	tfont_ary_show_list(stderr);
 
-int main(int argc, const char *argv[])
+	fprintf(stderr,
+		"[[ ENCODING ]]\n"
+#ifdef JFB_MINI_JFBTERM
+			" ISO-2022-JP EUC-JP"
+#else
+			" ISO-2022-*"
+#if defined(JFB_UNICODE) && defined(JFB_UTF_7)
+			" UTF-7"
+#endif	/* JFB_UNICODE && JFB_UTF_7 */
+#if defined(JFB_UNICODE) && defined(JFB_UTF_8)
+			" UTF-8"
+#endif	/* JFB_UNICODE && JFB_UTF_8 */
+#ifdef JFB_TRON
+			" TRON"
+#endif	/* JFB_TRON */
+#endif	/* JFB_MINI_JFBTERM */
+		"\n"
+		"[[ MISC ]]\n"
+#ifdef DEBUG
+			" DEBUG"
+#endif
+		"\n"
+	);
+
+}
+
+int main(int argc, char *argv[])
 {
 	TCapability* fcap;
 	char* cp;
 	const char* tn;
 	const char* en;
 
-	tapp_init(&gApp);
-
 	fprintf(stderr,
 		"%s - Kanji on framebuffer console Version %s\n"
-		"	Copyright (C) 1999 Noritoshi Masuichi\n"
+		"	Copyright (C) 1999-2000 Noritoshi Masuichi\n"
 		"This program is based on KON2\n"
 		"	Copyright (C) 1992-1996 Takashi MANABE\n\n",
-		PACKAGE, VERSION);
+		PACKAGE, VERSION
+	);
+
+	tapp_init(&gApp);
+
+	tapp_get_options(&gApp, argc, argv);
+
+	if (gApp.gOptShowHelpQ) {
+		ShowCaps();
+		exit(EXIT_SUCCESS);
+	}
 
 	if (geteuid() != 0) {
 		fprintf(stderr, "permission denied.\n");
