@@ -47,6 +47,7 @@
 #include <fbcommon.h>
 #include <font.h>
 #include <message.h>
+#include <main.h>
 
 
 int gChildProcessId = 0;
@@ -213,9 +214,6 @@ void tterm_start(TTerm* p, const char* tn, const char* en)
 
 void tterm_wakeup_shell_as_child(TTerm* p, const char* tn)
 {
-	char shell[128];
-	char *shell_name;
-
 	fflush(stdout);
 	gChildProcessId = fork();
 	if (gChildProcessId) {
@@ -245,16 +243,7 @@ void tterm_wakeup_shell_as_child(TTerm* p, const char* tn)
 	setgid(getgid());
 	setuid(getuid());
 
-	if (getenv("SHELL")) {
-		strcpy(shell, getenv("SHELL"));
-	} else {
-		strcpy(shell,"/bin/sh");
-	}
-	shell_name = shell;
-	if (shell[0] == '/') {
-		shell_name = strrchr(shell,'/') + 1;
-	}
-	execl(shell,shell_name,0);
+	execvp(gApp.gExecShell, gApp.gExecShellArgv);
 	exit(1);
 }
 
