@@ -51,6 +51,7 @@
 static void tvterm_set_default_encoding(TVterm* p, const char* en);
 static void tvterm_esc_start(TVterm* p, u_char ch);
 static void tvterm_esc_bracket(TVterm*, u_char);
+static void tvterm_esc_traditional_mulitbyte_fix(TVterm* p, u_char ch);
 static void tvterm_esc_desginate_font(TVterm* p, u_char ch);
 static void tvterm_invoke_gx(TVterm* p, TFontSpec* fs, u_int n);
 static void tvterm_re_invoke_gx(TVterm* p, TFontSpec* fs);
@@ -371,6 +372,8 @@ int tvterm_iso_C0_set(TVterm* p, u_char ch)
 		break;
 	case UNIVERSAL_ESC:
 		p->esc = tvterm_esc_start;
+		p->escSignature = 0;
+		p->escGn = 0;
 		return 1;
 	case ISO_LS1:
 		tvterm_invoke_gx(p, &(p->gl), 1); /* GL <== G1 */
@@ -435,7 +438,7 @@ void tvterm_emulate(TVterm* p, const char *buff, int nchars)
 
 #define ESC_ISO_GnDx(n, x) \
 	{\
-		p->escSignature = (x); p->escGn = (n);\
+		p->escSignature |= (x); p->escGn = (n);\
 		p->esc = tvterm_esc_desginate_font;\
 	}
 
@@ -449,7 +452,8 @@ static void tvterm_esc_start(TVterm* p, u_char ch)
 		p->esc = tvterm_esc_bracket;
 		break;
 	case ISO__MBS:
-		ESC_ISO_GnDx(0, TFONT_FT_94CHAR | TFONT_FT_DOUBLE);
+		p->esc = tvterm_esc_traditional_mulitbyte_fix;
+		p->escSignature = TFONT_FT_DOUBLE;
 		break;
 	case ISO_GZD4:
 		ESC_ISO_GnDx(0, TFONT_FT_94CHAR);
@@ -834,6 +838,15 @@ static void tvterm_esc_desginate_font(TVterm* p, u_char ch)
 	p->esc = NULL;
 }
 
+static void tvterm_esc_traditional_mulitbyte_fix(TVterm* p, u_char ch)
+{
+	if (ch == 0x40 || ch == 0x41 || ch == 0x42) {
+		tvterm_esc_desginate_font(p, ch);
+	} else {
+		tvterm_esc_start(p, ch);
+	}
+}
+
 #if 0
 static void tvterm_esc_set_double_code(TVterm* p, u_char ch)
 {
