@@ -350,6 +350,7 @@ void tvterm_refresh(TVterm* p)
 	u_int	x, y;
 	u_int lang;
 	u_char	ch, ch2;
+	u_int	chlw;
 	u_char	fc, bc, fg;
 	TFont* pf;
 	const u_char* glyph;
@@ -377,9 +378,10 @@ void tvterm_refresh(TVterm* p)
 			}
 			fc = p->attr[i] & 0xf;
 			bc = p->attr[i] >> 4;
-			ch2 = (p->text[i] >> 8) & 0xFF;
-			ch  = p->text[i] & 0xFF;
-			lang = (p->text[i] >> 24) & 0xFF;
+			chlw = p->text[i];
+			ch2 = (chlw >> 8) & 0xFF;
+			ch  = chlw & 0xFF;
+			lang = (chlw >> 24) & 0xFF;
 			p->flag[i] |= CLEAN_S;
 
 			if (fg & CODEIS_1) {
@@ -399,7 +401,7 @@ void tvterm_refresh(TVterm* p)
 				}
 #endif
 			}
-			glyph = pf->conv(pf, ch, ch2);
+			glyph = pf->conv(pf, chlw);
 			gFramebuffer.cap.fill(&gFramebuffer,
 				gFontsWidth * x, gFontsHeight * y,
 				gFontsWidth + (ch2 ? gFontsWidth : 0),
